@@ -106,6 +106,8 @@ extends 'AI::MXNet::Callback';
     frequent: int
         How many batches between calculations.
         Defaults to calculating & logging every 50 batches.
+    auto_reset: Bool
+        Reset the metric after each log, defaults to true.
 =cut
 
 has 'batch_size' => (is => 'ro', isa => 'Int', required => 1);
@@ -113,6 +115,7 @@ has 'frequent'   => (is => 'ro', isa => 'Int', default  => 50);
 has 'init'       => (is => 'rw', isa => 'Int', default  => 0);
 has 'tic'        => (is => 'rw', isa => 'Num', default  => 0);
 has 'last_count' => (is => 'rw', isa => 'Int', default  => 0);
+has 'auto_reset' => (is => 'ro', isa => 'Bool', default  => 1);
 
 method call(AI::MXNet::BatchEndParam $param)
 {
@@ -131,7 +134,7 @@ method call(AI::MXNet::BatchEndParam $param)
             if(defined $param->eval_metric)
             {
                 my $name_value = $param->eval_metric->get_name_value;
-                $param->eval_metric->reset;
+                $param->eval_metric->reset if $self->auto_reset;
                 while(my ($name, $value) = each %{ $name_value })
                 {
                     AI::MXNet::Logging->info(
@@ -221,7 +224,11 @@ package AI::MXNet::Callback;
 method Speedometer(@args)
 {
     AI::MXNet::Speedometer->new(
-        @args == 2 ? (batch_size => $args[0], frequent => $args[1]) : (batch_size => $args[0])
+        @args == 3 ?
+            (batch_size => $args[0], frequent => $args[1], auto_reset => $args[2])
+            : @args == 2 ?
+                (batch_size => $args[0], frequent => $args[1])
+                    : (batch_size => $args[0])
     )
 }
 
