@@ -232,13 +232,13 @@ method forward_backward(AI::MXNet::DataBatch $data_batch)
 =cut
 
 method score(
-    AI::MXNet::DataIter $eval_data,
-    EvalMetric          $eval_metric,
-    Maybe[Int]         :$num_batch=,
-    Maybe[Callback]    :$batch_end_callback=,
-    Maybe[Callback]    :$score_end_callback=,
-    Bool               :$reset=1,
-    Int                :$epoch=0
+    AI::MXNet::DataIter                 $eval_data,
+    EvalMetric                          $eval_metric,
+    Maybe[Int]                          :$num_batch=,
+    Maybe[Callback]|ArrayRef[Callback]  :$batch_end_callback=,
+    Maybe[Callback]|ArrayRef[Callback]  :$score_end_callback=,
+    Bool                                :$reset=1,
+    Int                                 :$epoch=0
 )
 {
     assert($self->binded and $self->params_initialized);
@@ -416,10 +416,10 @@ method predict(
         Default is 'accuracy'. The performance measure used to display during training.
         Other possible predefined metrics are:
         'ce' (CrossEntropy), 'f1', 'mae', 'mse', 'rmse', 'top_k_accuracy'
-    :$epoch_end_callback= : Maybe[Callback] function or array ref of functions.
+    :$epoch_end_callback= : Maybe[Callback]|ArrayRef[Callback] function or array ref of functions.
         Each callback will be called with the current $epoch, $symbol, $arg_params
         and $aux_params.
-    :$batch_end_callback= : Maybe[Callback] function or array ref of functions.
+    :$batch_end_callback= : Maybe[Callback]|ArrayRef[Callback] function or array ref of functions.
         Each callback will be called with a AI::MXNet::BatchEndParam.
     :$kvstore='local' : str or AI::MXNet::KVStore
         Default is 'local'.
@@ -428,10 +428,10 @@ method predict(
     :$optimizer_params : hash ref
         Default { learning_rate => 0.01 }.
         The parameters for the optimizer constructor.
-    :$eval_end_callback= : Maybe[Callback] function or array ref of functions
+    :$eval_end_callback= : Maybe[Callback]|ArrayRef[Callback] function or array ref of functions
         These will be called at the end of each full evaluation, with the metrics over
         the entire evaluation set.
-    :$eval_batch_end_callback : Maybe[Callback] function or array ref of functions
+    :$eval_batch_end_callback : Maybe[Callback]|ArrayRef[Callback] function or array ref of functions
         These will be called at the end of each minibatch during evaluation
     :$initializer= : Initializer
         Will be called to initialize the module parameters if not already initialized.
@@ -465,13 +465,13 @@ method fit(
     AI::MXNet::DataIter                 $train_data,
     Maybe[AI::MXNet::DataIter]         :$eval_data=,
     EvalMetric                         :$eval_metric='acc',
-    Maybe[Callback]                    :$epoch_end_callback=,
-    Maybe[Callback]                    :$batch_end_callback=,
+    Maybe[Callback]|ArrayRef[Callback] :$epoch_end_callback=,
+    Maybe[Callback]|ArrayRef[Callback] :$batch_end_callback=,
     Str                                :$kvstore='local',
     Optimizer                          :$optimizer='sgd',
     HashRef                            :$optimizer_params={ learning_rate => 0.01 },
-    Maybe[Callback]                    :$eval_end_callback=,
-    Maybe[Callback]                    :$eval_batch_end_callback=,
+    Maybe[Callback]|ArrayRef[Callback] :$eval_end_callback=,
+    Maybe[Callback]|ArrayRef[Callback] :$eval_batch_end_callback=,
     AI::MXNet::Initializer             :$initializer=AI::MXNet::Initializer->Uniform(scale => 0.01),
     Maybe[HashRef[AI::MXNet::NDArray]] :$arg_params=,
     Maybe[HashRef[AI::MXNet::NDArray]] :$aux_params=,
