@@ -732,6 +732,19 @@ method _get_ndarray_inputs(
     :$shapes : hash ref of str->Shape
         Input shape map, name->shape
 
+    :$shared_arg_names : Maybe[ArrayRef[Str]]
+        The argument names whose 'NDArray' of shared_exec can be reused for initializing
+        the current executor.
+
+    :$shared_exec : Maybe[AI::MXNet::Executor]
+        The executor whose arg_arrays, arg_arrays, grad_arrays, and aux_arrays can be
+        reused for initializing the current executor.
+
+    :$shared_buffer : Maybe[HashRef[AI::MXNet::NDArray]]
+        The dict mapping argument names to the `NDArray` that can be reused for initializing
+        the current executor. This buffer will be checked for reuse if one argument name
+        of the current executor is not found in `shared_arg_names`.
+
     Returns
     -------
     $executor : AI::MXNet::Executor
@@ -739,115 +752,161 @@ method _get_ndarray_inputs(
 =cut
 
 method simple_bind(
-            AI::MXNet::Context                 :$ctx=AI::MXNet::Context->current_ctx,
-            Maybe[HashRef[Shape]]              :$shapes=,
-            Str|HashRef[Str]                   :$grad_req='write',
-            Maybe[HashRef[Dtype]]              :$type_dict=,
-            Maybe[HashRef[AI::MXNet::Context]] :$group2ctx=
+    AI::MXNet::Context                             :$ctx=AI::MXNet::Context->current_ctx,
+    GradReq|ArrayRef[GradReq]|HashRef[GradReq]     :$grad_req='write',
+    Maybe[HashRef[Shape]]                          :$shapes=,
+    Maybe[HashRef[Dtype]]                          :$type_dict=,
+    Maybe[HashRef[AI::MXNet::Context]]             :$group2ctx=,
+    Maybe[ArrayRef[Str]]                           :$shared_arg_names=,
+    Maybe[AI::MXNet::Executor]                     :$shared_exec=,
+    Maybe[HashRef[AI::MXNet::NDArray]]             :$shared_buffer=
 )
 {
-    $shapes //= {};
-    if(not defined $type_dict)
+    my $num_provided_arg_types;
+    my @provided_arg_type_names;
+    my @provided_arg_type_data;
+    if(defined $type_dict)
     {
-        $type_dict =  {};
-        my $attrs = $self->attr_dict;
-        for my $k (@{ $self->list_arguments })
+        while(my ($k, $v) = each %{ $type_dict })
         {
-            if(not exists $attrs->{$k} or not exists $attrs->{$k}{__dtype__})
+            push @provided_arg_type_names, $k;
+            push @provided_arg_type_data, DTYPE_STR_TO_MX->{$v};
+        }
+        $num_provided_arg_types = @provided_arg_type_names;
+    }
+    my @provided_arg_shape_data;
+    # argument shape index in sdata,
+    # e.g. [sdata[indptr[0]], sdata[indptr[1]]) is the shape of the first arg
+    my @provided_arg_shape_idx = (0);
+    my @provided_arg_shape_names;
+    while(my ($k, $v) = each %{ $shapes//{} })
+    {
+        push @provided_arg_shape_names, $k;
+        push @provided_arg_shape_data, @{ $v };
+        push @provided_arg_shape_idx, scalar(@provided_arg_shape_data);
+    }
+    $num_provided_arg_types = @provided_arg_type_names;
+
+    my $provided_req_type_list_len = 0;
+    my @provided_grad_req_types;
+    my @provided_grad_req_names;
+    if(defined $grad_req)
+    {
+        if(not ref $grad_req)
+        {
+            push @provided_grad_req_types, $grad_req;
+        }
+        elsif(ref $grad_req eq 'ARRAY')
+        {
+            assert((@{ $grad_req } != 0), 'grad_req in simple_bind cannot be an empty list');
+            @provided_grad_req_types = @{ $grad_req };
+            $provided_req_type_list_len = @provided_grad_req_types;
+        }
+        elsif(ref $grad_req eq 'HASH')
+        {
+            assert((keys %{ $grad_req } != 0), 'grad_req in simple_bind cannot be an empty hash');
+            while(my ($k, $v) = each %{ $grad_req })
             {
-                $type_dict->{ $k } = 'float32';
+                push @provided_grad_req_names, $k;
+                push @provided_grad_req_types, $v;
             }
+            $provided_req_type_list_len = @provided_grad_req_types;
         }
     }
-    my @keys = keys %$shapes;
-    my @shape_input;
-    my @type_input;
-    for my $k (@keys)
-    {
-        push @shape_input, ($k => $shapes->{$k});
-        push @type_input,  ($k => $type_dict->{$k})
-    }
-    my ($arg_shapes, undef, $aux_shapes) = $self->infer_shape(@shape_input);
-    my ($arg_types,  undef, $aux_types)  = $self->infer_type(@type_input);
-    confess("Input node is not complete") 
-        unless $arg_shapes and $arg_types;
-
-    my ($arg_ctx, $aux_ctx) = ([], []);
+    my $num_ctx_map_keys = 0;
+    my @ctx_map_keys;
+    my @ctx_map_dev_types;
+    my @ctx_map_dev_ids;
     if(defined $group2ctx)
     {
-        my $attr_dict = $self->attr_dict();
-        for my $name (@{ $self->list_arguments() })
+        while(my ($k, $v) = each %{ $group2ctx })
         {
-            if(
-                exists $attr_dict->{ $name }
-                    and
-                exists $attr_dict->{ $name }{ __ctx_group__ }
-                    and
-                $group2ctx->{ $attr_dict->{ $name }{ __ctx_group__ } }
-            )
-            {
-                push @{ $arg_ctx }, $group2ctx->{ $attr_dict->{ $name }{ __ctx_group__ } };
-            }
-            else
-            {
-                push @{ $arg_ctx }, $ctx;
-            }
+            push @ctx_map_keys, $k;
+            push @ctx_map_dev_types, $v->device_type_id;
+            push @ctx_map_dev_ids, $v->device_id;
         }
-        for my $name (@{ $self->list_auxiliary_states() })
+        $num_ctx_map_keys = @ctx_map_keys;
+    }
+
+    my @shared_arg_name_list;
+    if(defined $shared_arg_names)
+    {
+        @shared_arg_name_list = @{ $shared_arg_names };
+    }
+    my %shared_data;
+    if(defined $shared_buffer)
+    {
+        while(my ($k, $v) = each %{ $shared_buffer })
         {
-            if(
-                exists $attr_dict->{ $name }
-                    and
-                exists $attr_dict->{ $name }{ __ctx_group__ }
-                    and
-                $group2ctx->{ $attr_dict->{ $name }{ __ctx_group__ } }
-            )
-            {
-                push @{ $aux_ctx }, $group2ctx->{ $attr_dict->{ $name }{ __ctx_group__ } };
-            }
-            else
-            {
-                push @{ $aux_ctx }, $ctx;
-            }
+            $shared_data{$k} = $v->handle;
         }
     }
-    else
-    {
-        @{ $arg_ctx } = (($ctx) x @{ $arg_shapes });
-        @{ $aux_ctx } = (($ctx) x @{ $aux_shapes });
-    }
-    my @arg_ndarrays;
-    for (my $i = 0; $i < @{ $arg_types }; $i++)
-    {
-        push @arg_ndarrays, AI::MXNet::NDArray->zeros(
-            $arg_shapes->[$i], ctx => $arg_ctx->[$i], dtype => $arg_types->[$i]
-        );
-    }
-    my $grad_ndarrays;
-    if($grad_req ne 'null')
-    {
-        my $names = $self->list_arguments;
-        for (my $i = 0; $i < @{ $arg_types }; $i++)
-        {
-            if(not ref $grad_req eq 'HASH' or not ($grad_req->{ $names->[$i] }//'') eq 'null')
-            {
-                $grad_ndarrays->{ $names->[$i] } = AI::MXNet::NDArray->zeros(
-                    $arg_shapes->[$i], ctx => $arg_ctx->[$i], dtype => $arg_types->[$i]
-                );
-            }
-        }
-    }
-    my @aux_ndarrays;
-    for (my $i = 0; $i < @{ $aux_types }; $i++)
-    {
-        push @aux_ndarrays, AI::MXNet::NDArray->zeros(
-            $aux_shapes->[$i], ctx => $aux_ctx->[$i], dtype => $aux_types->[$i]
-        );
-    }
-    my $executor = $self->bind(
-        ctx => $ctx, args => \@arg_ndarrays, args_grad => $grad_ndarrays,
-        grad_req => $grad_req, aux_states => \@aux_ndarrays, group2ctx => $group2ctx
+    my $shared_exec_handle = defined $shared_exec ? $shared_exec->handle : undef;
+    my (
+        $updated_shared_data,
+        $in_arg_handles,
+        $arg_grad_handles,
+        $aux_state_handles,
+        $exe_handle
     );
+    eval {
+        ($updated_shared_data, $in_arg_handles, $arg_grad_handles, $aux_state_handles, $exe_handle)
+            =
+        check_call(
+            AI::MXNetCAPI::ExecutorSimpleBind(
+                $self->handle,
+                $ctx->device_type_id,
+                $ctx->device_id,
+                $num_ctx_map_keys,
+                \@ctx_map_keys,
+                \@ctx_map_dev_types,
+                \@ctx_map_dev_ids,
+                $provided_req_type_list_len,
+                \@provided_grad_req_names,
+                \@provided_grad_req_types,
+                scalar(@provided_arg_shape_names),
+                \@provided_arg_shape_names,
+                \@provided_arg_shape_data,
+                \@provided_arg_shape_idx,
+                $num_provided_arg_types,
+                \@provided_arg_type_names,
+                \@provided_arg_type_data,
+                scalar(@shared_arg_name_list),
+                \@shared_arg_name_list,
+                defined $shared_buffer ? \%shared_data : undef,
+                $shared_exec_handle
+            )
+        );
+    };
+    if($@)
+    {
+        confess(
+            "simple_bind failed: Error: $@; Arguments: ".
+            Data::Dumper->new(
+                [$shapes//{}]
+            )->Purity(1)->Deepcopy(1)->Terse(1)->Dump
+        );
+    }
+    if(defined $shared_buffer)
+    {
+        while(my ($k, $v) = each %{ $updated_shared_data })
+        {
+            $shared_buffer->{$k} = AI::MXNet::NDArray->new(handle => $v);
+        }
+    }
+    my @arg_arrays  = map { AI::MXNet::NDArray->new(handle => $_) } @{ $in_arg_handles };
+    my @grad_arrays = map { defined $_ ? AI::MXNet::NDArray->new(handle => $_) : undef  } @{ $arg_grad_handles };
+    my @aux_arrays  = map { AI::MXNet::NDArray->new(handle => $_) } @{ $aux_state_handles };
+    my $executor = AI::MXNet::Executor->new(
+        handle    => $exe_handle,
+        symbol    => $self,
+        ctx       => $ctx,
+        grad_req  => $grad_req,
+        group2ctx => $group2ctx
+    );
+    $executor->arg_arrays(\@arg_arrays);
+    $executor->grad_arrays(\@grad_arrays);
+    $executor->aux_arrays(\@aux_arrays);
     return $executor;
 }
 
@@ -1288,6 +1347,8 @@ method arange(Index :$start=0, Index :$stop=, Num :$step=1.0, Index :$repeat=1,
     });
 }
 
+method CachedOp(@args)        { AI::MXNet::CachedOp->new(@args) }
+
 sub _parse_arguments
 {
     my $type = shift;
